import { Query, QueryOptions } from './query';
/**
 * Creates a new `Query` which maps a source value by the provided mapping
 * function.
 *
 * @param query source query
 * @param mapper value mapper
 * @param options options for processing the result value
 */
export declare function mapQuery<T, R, K = R>(query: Query<T>, mapper: (value: T) => R, options?: QueryOptions<R, K>): Query<R>;
/**
 * Creates a new `Query` which takes the latest values from source queries
 * and merges them into a single value.
 *
 * @param queries source queries
 * @param merger value merger
 * @param options options for processing the result value
 */
export declare function mergeQueries<Values extends unknown[], Result, ResultKey = Result>(queries: [
    ...{
        [K in keyof Values]: Query<Values[K]>;
    }
], merger: (...values: Values) => Result, options?: QueryOptions<Result, ResultKey>): Query<Result>;
